from pydantic import Field
from .base_model import BaseModel
from .fragments import JobData

class CreateJob(BaseModel):
    """@public"""
    create_job: 'CreateJobCreateJob' = Field(alias='createJob')

class CreateJobCreateJob(JobData):
    """@public"""
    pass
CreateJob.model_rebuild()


from typing import Any, TypeVar, Generic
from pydantic import BaseModel as PydanticBaseModel, model_validator, ConfigDict


class BaseModel(PydanticBaseModel):
    model_config = ConfigDict(extra="forbid")


MetadataType = TypeVar("MetadataType", bound=BaseModel)


class Turn(BaseModel):
    """@public"""

    role: str
    content: str


class Request(BaseModel):
    """@public"""

    turns: list[Turn]
    metadata: dict[str, Any] | None = None
    id: int | None = None


class BatchedRequest(BaseModel):
    """@public"""

    requests: list[Request]


class ValidatedRequest(BaseModel, Generic[MetadataType]):
    """@public"""

    turns: list[Turn]
    metadata: MetadataType
    id: int | None = None

    @model_validator(mode="before")
    @classmethod
    def intercept_none_metadata(cls, data: Any) -> Any:
        # we need an empty dict to map to an empty pydantic base model in the server
        if "metadata" in data and data["metadata"] is None:
            data["metadata"] = {}
        return data


class ValidatedBatchedRequest(BaseModel, Generic[MetadataType]):
    """@public"""

    requests: list[ValidatedRequest[MetadataType]]


class Response(BaseModel):
    """@public"""

    reward: float
    metadata: dict[str, Any]
    id: int | None = None


class BatchedResponse(BaseModel):
    """@public"""

    responses: list[Response]


class MetadataValidationResponse(BaseModel):
    """@public"""

    is_valid: bool
    error_message: str | None = None


class BatchedMetadataValidationResponse(BaseModel):
    """@public"""

    responses: list[MetadataValidationResponse]


class ServerInfo(BaseModel):
    """@public"""

    version: str
    name: str
    description: str



from .base_operation import GraphQLField

class AbReportGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'AbReportGraphQLField':
        self._alias = alias
        return self

class AbVariantReportGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'AbVariantReportGraphQLField':
        self._alias = alias
        return self

class AbVariantReportComparisonGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'AbVariantReportComparisonGraphQLField':
        self._alias = alias
        return self

class AbcampaignGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'AbcampaignGraphQLField':
        self._alias = alias
        return self

class ActivityGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'ActivityGraphQLField':
        self._alias = alias
        return self

class ActivityOutputGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'ActivityOutputGraphQLField':
        self._alias = alias
        return self

class ApiKeyGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'ApiKeyGraphQLField':
        self._alias = alias
        return self

class ArtifactByproductsUnion(GraphQLField):
    """@private"""

    def on(self, type_name: str, *subfields: GraphQLField) -> 'ArtifactByproductsUnion':
        self._inline_fragments[type_name] = subfields
        return self

    def alias(self, alias: str) -> 'ArtifactByproductsUnion':
        self._alias = alias
        return self

class AuthProviderGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'AuthProviderGraphQLField':
        self._alias = alias
        return self

class BatchInferenceJobStageOutputGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'BatchInferenceJobStageOutputGraphQLField':
        self._alias = alias
        return self

class BillingUsageGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'BillingUsageGraphQLField':
        self._alias = alias
        return self

class BufferInfoGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'BufferInfoGraphQLField':
        self._alias = alias
        return self

class ChatMessageGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'ChatMessageGraphQLField':
        self._alias = alias
        return self

class ComparisonFeedbackGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'ComparisonFeedbackGraphQLField':
        self._alias = alias
        return self

class CompletionGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'CompletionGraphQLField':
        self._alias = alias
        return self

class CompletionConnectionGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'CompletionConnectionGraphQLField':
        self._alias = alias
        return self

class CompletionEdgeGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'CompletionEdgeGraphQLField':
        self._alias = alias
        return self

class CompletionGroupDataGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'CompletionGroupDataGraphQLField':
        self._alias = alias
        return self

class CompletionGroupDataConnectionGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'CompletionGroupDataConnectionGraphQLField':
        self._alias = alias
        return self

class CompletionGroupDataEdgeGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'CompletionGroupDataEdgeGraphQLField':
        self._alias = alias
        return self

class CompletionGroupFeedbackStatsGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'CompletionGroupFeedbackStatsGraphQLField':
        self._alias = alias
        return self

class CompletionHistoryEntryOuputGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'CompletionHistoryEntryOuputGraphQLField':
        self._alias = alias
        return self

class CompletionLabelGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'CompletionLabelGraphQLField':
        self._alias = alias
        return self

class CompletionMetadataGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'CompletionMetadataGraphQLField':
        self._alias = alias
        return self

class ComputePoolGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'ComputePoolGraphQLField':
        self._alias = alias
        return self

class ContractGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'ContractGraphQLField':
        self._alias = alias
        return self

class CustomConfigOutputGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'CustomConfigOutputGraphQLField':
        self._alias = alias
        return self

class CustomRecipeGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'CustomRecipeGraphQLField':
        self._alias = alias
        return self

class CustomRecipeJobDetailsGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'CustomRecipeJobDetailsGraphQLField':
        self._alias = alias
        return self

class DatasetGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'DatasetGraphQLField':
        self._alias = alias
        return self

class DatasetByproductsGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'DatasetByproductsGraphQLField':
        self._alias = alias
        return self

class DatasetMetricUsageGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'DatasetMetricUsageGraphQLField':
        self._alias = alias
        return self

class DatasetProgressGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'DatasetProgressGraphQLField':
        self._alias = alias
        return self

class DatasetUploadProcessingStatusGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'DatasetUploadProcessingStatusGraphQLField':
        self._alias = alias
        return self

class DatasetValidationOutputGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'DatasetValidationOutputGraphQLField':
        self._alias = alias
        return self

class DeleteConfirmGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'DeleteConfirmGraphQLField':
        self._alias = alias
        return self

class DirectFeedbackGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'DirectFeedbackGraphQLField':
        self._alias = alias
        return self

class EmojiGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'EmojiGraphQLField':
        self._alias = alias
        return self

class EvalJobStageOutputGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'EvalJobStageOutputGraphQLField':
        self._alias = alias
        return self

class EvaluationByproductsGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'EvaluationByproductsGraphQLField':
        self._alias = alias
        return self

class EvaluationResultGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'EvaluationResultGraphQLField':
        self._alias = alias
        return self

class GlobalUsageGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'GlobalUsageGraphQLField':
        self._alias = alias
        return self

class GpuAllocationGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'GpuAllocationGraphQLField':
        self._alias = alias
        return self

class GraderGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'GraderGraphQLField':
        self._alias = alias
        return self

class GraderConfigUnion(GraphQLField):
    """@private"""

    def on(self, type_name: str, *subfields: GraphQLField) -> 'GraderConfigUnion':
        self._inline_fragments[type_name] = subfields
        return self

    def alias(self, alias: str) -> 'GraderConfigUnion':
        self._alias = alias
        return self

class HarmonyGroupGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'HarmonyGroupGraphQLField':
        self._alias = alias
        return self

class HarmonyGroupMetricEventUnion(GraphQLField):
    """@private"""

    def on(self, type_name: str, *subfields: GraphQLField) -> 'HarmonyGroupMetricEventUnion':
        self._inline_fragments[type_name] = subfields
        return self

    def alias(self, alias: str) -> 'HarmonyGroupMetricEventUnion':
        self._alias = alias
        return self

class HarmonyMemoryUsageGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'HarmonyMemoryUsageGraphQLField':
        self._alias = alias
        return self

class HarmonyMidMetricsGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'HarmonyMidMetricsGraphQLField':
        self._alias = alias
        return self

class InteractionOutputGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'InteractionOutputGraphQLField':
        self._alias = alias
        return self

class IntervalGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'IntervalGraphQLField':
        self._alias = alias
        return self

class JobGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'JobGraphQLField':
        self._alias = alias
        return self

class JobArtifactGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'JobArtifactGraphQLField':
        self._alias = alias
        return self

class JobConnectionGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'JobConnectionGraphQLField':
        self._alias = alias
        return self

class JobEdgeGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'JobEdgeGraphQLField':
        self._alias = alias
        return self

class JobStageInfoOutputUnion(GraphQLField):
    """@private"""

    def on(self, type_name: str, *subfields: GraphQLField) -> 'JobStageInfoOutputUnion':
        self._inline_fragments[type_name] = subfields
        return self

    def alias(self, alias: str) -> 'JobStageInfoOutputUnion':
        self._alias = alias
        return self

class JobStageOutputGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'JobStageOutputGraphQLField':
        self._alias = alias
        return self

class JudgeGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'JudgeGraphQLField':
        self._alias = alias
        return self

class JudgeConfigOutputGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'JudgeConfigOutputGraphQLField':
        self._alias = alias
        return self

class JudgeExampleGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'JudgeExampleGraphQLField':
        self._alias = alias
        return self

class LabelGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'LabelGraphQLField':
        self._alias = alias
        return self

class LabelKeyUsageGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'LabelKeyUsageGraphQLField':
        self._alias = alias
        return self

class LabelUsageGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'LabelUsageGraphQLField':
        self._alias = alias
        return self

class LabelValueUsageGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'LabelValueUsageGraphQLField':
        self._alias = alias
        return self

class MetaObjectGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'MetaObjectGraphQLField':
        self._alias = alias
        return self

class MetricGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'MetricGraphQLField':
        self._alias = alias
        return self

class MetricActivityGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'MetricActivityGraphQLField':
        self._alias = alias
        return self

class MetricWithContextGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'MetricWithContextGraphQLField':
        self._alias = alias
        return self

class ModelGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'ModelGraphQLField':
        self._alias = alias
        return self

class ModelByproductsGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'ModelByproductsGraphQLField':
        self._alias = alias
        return self

class ModelComputeConfigOutputGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'ModelComputeConfigOutputGraphQLField':
        self._alias = alias
        return self

class ModelPlacementOutputGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'ModelPlacementOutputGraphQLField':
        self._alias = alias
        return self

class ModelServiceGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'ModelServiceGraphQLField':
        self._alias = alias
        return self

class MutationRootGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'MutationRootGraphQLField':
        self._alias = alias
        return self

class PageInfoGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'PageInfoGraphQLField':
        self._alias = alias
        return self

class PrebuiltConfigDefinitionGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'PrebuiltConfigDefinitionGraphQLField':
        self._alias = alias
        return self

class PrebuiltConfigOutputGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'PrebuiltConfigOutputGraphQLField':
        self._alias = alias
        return self

class PrebuiltCriteriaGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'PrebuiltCriteriaGraphQLField':
        self._alias = alias
        return self

class ProviderListGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'ProviderListGraphQLField':
        self._alias = alias
        return self

class QueryRootGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'QueryRootGraphQLField':
        self._alias = alias
        return self

class RemoteConfigOutputGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'RemoteConfigOutputGraphQLField':
        self._alias = alias
        return self

class RemoteEnvGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'RemoteEnvGraphQLField':
        self._alias = alias
        return self

class RemoteEnvTestUnion(GraphQLField):
    """@private"""

    def on(self, type_name: str, *subfields: GraphQLField) -> 'RemoteEnvTestUnion':
        self._inline_fragments[type_name] = subfields
        return self

    def alias(self, alias: str) -> 'RemoteEnvTestUnion':
        self._alias = alias
        return self

class RemoteEnvTestOfflineGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'RemoteEnvTestOfflineGraphQLField':
        self._alias = alias
        return self

class RemoteEnvTestOnlineGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'RemoteEnvTestOnlineGraphQLField':
        self._alias = alias
        return self

class RoleGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'RoleGraphQLField':
        self._alias = alias
        return self

class SearchResultGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'SearchResultGraphQLField':
        self._alias = alias
        return self

class SessionGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'SessionGraphQLField':
        self._alias = alias
        return self

class SettingsGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'SettingsGraphQLField':
        self._alias = alias
        return self

class ShareGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'ShareGraphQLField':
        self._alias = alias
        return self

class SubscriptionRootGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'SubscriptionRootGraphQLField':
        self._alias = alias
        return self

class SystemPromptTemplateGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'SystemPromptTemplateGraphQLField':
        self._alias = alias
        return self

class TeamGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'TeamGraphQLField':
        self._alias = alias
        return self

class TeamMemberGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'TeamMemberGraphQLField':
        self._alias = alias
        return self

class TeamWithroleGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'TeamWithroleGraphQLField':
        self._alias = alias
        return self

class TimeseriesGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'TimeseriesGraphQLField':
        self._alias = alias
        return self

class ToolProviderGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'ToolProviderGraphQLField':
        self._alias = alias
        return self

class TrainingJobStageOutputGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'TrainingJobStageOutputGraphQLField':
        self._alias = alias
        return self

class TrendResultGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'TrendResultGraphQLField':
        self._alias = alias
        return self

class UnitConfigGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'UnitConfigGraphQLField':
        self._alias = alias
        return self

class UsageGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'UsageGraphQLField':
        self._alias = alias
        return self

class UsageAggregateItemGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'UsageAggregateItemGraphQLField':
        self._alias = alias
        return self

class UsageAggregatePerUseCaseItemGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'UsageAggregatePerUseCaseItemGraphQLField':
        self._alias = alias
        return self

class UsageStatsGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'UsageStatsGraphQLField':
        self._alias = alias
        return self

class UsageStatsByModelGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'UsageStatsByModelGraphQLField':
        self._alias = alias
        return self

class UseCaseGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'UseCaseGraphQLField':
        self._alias = alias
        return self

class UseCaseItemGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'UseCaseItemGraphQLField':
        self._alias = alias
        return self

class UseCaseMetadataGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'UseCaseMetadataGraphQLField':
        self._alias = alias
        return self

class UserGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'UserGraphQLField':
        self._alias = alias
        return self

class WidgetGraphQLField(GraphQLField):
    """@private"""

    def alias(self, alias: str) -> 'WidgetGraphQLField':
        self._alias = alias
        return self


from __future__ import annotations
from typing import TYPE_CHECKING, List

from adaptive_sdk.graphql_client import ListRolesRoles, RoleCreate, CreateRoleCreateRole

from .base_resource import SyncAPIResource, AsyncAPIResource, UseCaseResource

if TYPE_CHECKING:
    from adaptive_sdk.client import Adaptive, AsyncAdaptive


class Roles(SyncAPIResource, UseCaseResource):  # type: ignore[misc]
    """
    Resource to manage roles.
    """

    def __init__(self, client: Adaptive) -> None:
        SyncAPIResource.__init__(self, client)
        UseCaseResource.__init__(self, client)

    def list(self) -> List[ListRolesRoles]:
        return self._gql_client.list_roles().roles

    def create(
        self, key: str, permissions: List[str], name: str | None = None
    ) -> CreateRoleCreateRole:
        """
        Creates new role.

        Args:
            key: Role key.
            permissions: List of permission identifiers such as `use_case:read`. You can list all possible permissions with client.permissions.list().
            name: Role name; if not provided, defaults to `key`.
        """
        input = RoleCreate(key=key, name=name or key, permissions=permissions)
        return self._gql_client.create_role(input).create_role


class AsyncRoles(AsyncAPIResource, UseCaseResource):  # type: ignore[misc]
    """
    Resource to manage roles.
    """

    def __init__(self, client: AsyncAdaptive) -> None:
        AsyncAPIResource.__init__(self, client)
        UseCaseResource.__init__(self, client)

    async def list(self) -> List[ListRolesRoles]:
        return (await self._gql_client.list_roles()).roles

    async def create(
        self, key: str, permissions: List[str], name: str | None = None
    ) -> CreateRoleCreateRole:
        """
        Creates new role.

        Args:
            key: Role key.
            permissions: List of permission identifiers such as `use_case:read`. You can list all possible permissions with client.permissions.list().
            name: Role name; if not provided, defaults to `key`.
        """
        input = RoleCreate(key=key, name=name or key, permissions=permissions)
        return (await self._gql_client.create_role(input)).create_role



from pydantic import Field
from .base_model import BaseModel
from .fragments import RemoteEnvData

class AddRemoteEnv(BaseModel):
    """@public"""
    add_remote_env: 'AddRemoteEnvAddRemoteEnv' = Field(alias='addRemoteEnv')

class AddRemoteEnvAddRemoteEnv(RemoteEnvData):
    """@public"""
    pass
AddRemoteEnv.model_rebuild()


from enum import Enum

class AbcampaignStatus(str, Enum):
    """@public"""
    WARMUP = 'WARMUP'
    IN_PROGRESS = 'IN_PROGRESS'
    DONE = 'DONE'
    CANCELLED = 'CANCELLED'

class AuthProviderKind(str, Enum):
    """@public"""
    OIDC = 'OIDC'

class BillingCycle(str, Enum):
    """@public"""
    YEARLY = 'YEARLY'

class CompletionGroupBy(str, Enum):
    """@public"""
    MODEL = 'MODEL'
    PROMPT = 'PROMPT'

class CompletionSource(str, Enum):
    """@public"""
    LIVE = 'LIVE'
    OFFLINE = 'OFFLINE'
    AUTOMATION = 'AUTOMATION'
    DATASET = 'DATASET'
    EVALUATION = 'EVALUATION'

class ComputePoolCapability(str, Enum):
    """@public"""
    INFERENCE = 'INFERENCE'
    JOB = 'JOB'

class DatasetKind(str, Enum):
    """@public"""
    PROMPT = 'PROMPT'
    PROMPT_COMPLETION = 'PROMPT_COMPLETION'
    PROMPT_COMPLETION_FEEDBACK = 'PROMPT_COMPLETION_FEEDBACK'
    PREFERENCE = 'PREFERENCE'

class DatasetSource(str, Enum):
    """@public"""
    UPLOADED = 'UPLOADED'
    GENERATED = 'GENERATED'
    GENERATING = 'GENERATING'
    FILTERED = 'FILTERED'

class DatasetStatus(str, Enum):
    """@public"""
    PENDING = 'PENDING'
    PROCESSING = 'PROCESSING'
    READY = 'READY'
    FAILED = 'FAILED'

class DateBucketUnit(str, Enum):
    """@public"""
    HOUR = 'HOUR'
    DAY = 'DAY'
    WEEK = 'WEEK'
    MONTH = 'MONTH'
    QUARTER = 'QUARTER'
    YEAR = 'YEAR'
    NO_GROUP_BY_DATE = 'NO_GROUP_BY_DATE'

class ExternalModelProviderName(str, Enum):
    """@public"""
    OPEN_AI = 'OPEN_AI'
    GOOGLE = 'GOOGLE'
    ANTHROPIC = 'ANTHROPIC'

class FeedbackType(str, Enum):
    """@public"""
    DIRECT = 'DIRECT'
    COMPARISON = 'COMPARISON'

class GraderTypeEnum(str, Enum):
    """@public"""
    JUDGE = 'JUDGE'
    PREBUILT = 'PREBUILT'
    REMOTE = 'REMOTE'
    CUSTOM = 'CUSTOM'

class HarmonyStatus(str, Enum):
    """@public"""
    ONLINE = 'ONLINE'
    OFFLINE = 'OFFLINE'

class JobArtifactKind(str, Enum):
    """@public"""
    EVALUATION = 'EVALUATION'
    CUSTOM = 'CUSTOM'
    DATASET = 'DATASET'
    MODEL = 'MODEL'

class JobKind(str, Enum):
    """@public"""
    TRAINING = 'TRAINING'
    EVALUATION = 'EVALUATION'
    DATASET_GENERATION = 'DATASET_GENERATION'
    MODEL_CONVERSION = 'MODEL_CONVERSION'
    CUSTOM = 'CUSTOM'

class JobStatus(str, Enum):
    """@public"""
    PENDING = 'PENDING'
    RUNNING = 'RUNNING'
    COMPLETED = 'COMPLETED'
    FAILED = 'FAILED'
    CANCELED = 'CANCELED'

class JobStatusOutput(str, Enum):
    """@public"""
    PENDING = 'PENDING'
    RUNNING = 'RUNNING'
    DONE = 'DONE'
    CANCELLED = 'CANCELLED'
    ERROR = 'ERROR'

class JudgeCapability(str, Enum):
    """@public"""
    TRAINING = 'TRAINING'
    EVALUATION = 'EVALUATION'

class MetricAggregation(str, Enum):
    """@public"""
    AVERAGE = 'AVERAGE'
    SUM = 'SUM'
    COUNT = 'COUNT'

class MetricKind(str, Enum):
    """@public"""
    SCALAR = 'SCALAR'
    BOOL = 'BOOL'

class MetricScoringType(str, Enum):
    """@public"""
    HIGHER_IS_BETTER = 'HIGHER_IS_BETTER'
    LOWER_IS_BETTER = 'LOWER_IS_BETTER'

class ModelCapabilityFilter(str, Enum):
    """@public"""
    Embedding = 'Embedding'
    TextGeneration = 'TextGeneration'
    ImageUnderstanding = 'ImageUnderstanding'
    Reasoning = 'Reasoning'
    Regression = 'Regression'

class ModelOnline(str, Enum):
    """@public"""
    ONLINE = 'ONLINE'
    PENDING = 'PENDING'
    OFFLINE = 'OFFLINE'
    ERROR = 'ERROR'

class ModelserviceStatus(str, Enum):
    """@public"""
    PENDING = 'PENDING'
    ONLINE = 'ONLINE'
    OFFLINE = 'OFFLINE'
    DETACHED = 'DETACHED'
    TURNED_OFF = 'TURNED_OFF'
    ERROR = 'ERROR'

class OpenAIModel(str, Enum):
    """@public"""
    GPT41 = 'GPT41'
    GPT4O = 'GPT4O'
    GPT4O_MINI = 'GPT4O_MINI'
    O1 = 'O1'
    O1_MINI = 'O1_MINI'
    O3_MINI = 'O3_MINI'
    O4_MINI = 'O4_MINI'
    O3 = 'O3'
    GPT4 = 'GPT4'
    GPT4_TURBO = 'GPT4_TURBO'
    GPT3_5_TURBO = 'GPT3_5_TURBO'
    GPT5 = 'GPT5'
    GPT5_MINI = 'GPT5_MINI'
    GPT5_NANO = 'GPT5_NANO'
    GPT41_MINI = 'GPT41_MINI'

class PrebuiltCriteriaKey(str, Enum):
    """@public"""
    FAITHFULNESS = 'FAITHFULNESS'
    ANSWER_RELEVANCY = 'ANSWER_RELEVANCY'
    CONTEXT_RELEVANCY = 'CONTEXT_RELEVANCY'

class Protocol(str, Enum):
    """@public"""
    HTTP = 'HTTP'

class ProviderName(str, Enum):
    """@public"""
    OPEN_AI = 'OPEN_AI'
    HARMONY = 'HARMONY'
    GOOGLE = 'GOOGLE'
    ANTHROPIC = 'ANTHROPIC'

class RemoteEnvStatus(str, Enum):
    """@public"""
    ONLINE = 'ONLINE'
    OFFLINE = 'OFFLINE'

class SelectionTypeInput(str, Enum):
    """@public"""
    ALL = 'ALL'
    RANDOM = 'RANDOM'
    LAST = 'LAST'

class SessionStatus(str, Enum):
    """@public"""
    PENDING = 'PENDING'
    DONE = 'DONE'
    ERROR = 'ERROR'

class SortDirection(str, Enum):
    """@public"""
    ASC = 'ASC'
    DESC = 'DESC'

class TimeseriesInterval(str, Enum):
    """@public"""
    HOUR = 'HOUR'
    DAY = 'DAY'
    WEEK = 'WEEK'
    MONTH = 'MONTH'
    QUARTER = 'QUARTER'
    YEAR = 'YEAR'
    MILLENNIUM = 'MILLENNIUM'

class UnitPosition(str, Enum):
    """@public"""
    LEFT = 'LEFT'
    RIGHT = 'RIGHT'


from typing import List, Optional
from .base_model import BaseModel
from .fragments import ModelDataAdmin

class ListModels(BaseModel):
    """@public"""
    models: List['ListModelsModels']

class ListModelsModels(ModelDataAdmin):
    """@public"""
    backbone: Optional['ListModelsModelsBackbone']

class ListModelsModelsBackbone(ModelDataAdmin):
    """@public"""
    pass
ListModels.model_rebuild()
ListModelsModels.model_rebuild()


from typing import List
from pydantic import Field
from .base_model import BaseModel
from .fragments import JudgeData

class ListJudgeVersions(BaseModel):
    """@public"""
    judge_versions: List['ListJudgeVersionsJudgeVersions'] = Field(alias='judgeVersions')

class ListJudgeVersionsJudgeVersions(JudgeData):
    """@public"""
    pass
ListJudgeVersions.model_rebuild()


from __future__ import annotations
from typing import get_args, Callable, Literal, Sequence, TYPE_CHECKING

from adaptive_sdk.graphql_client import (
    OpenAIModel,
    OpenAIProviderDataInput,
    GoogleProviderDataInput,
    ModelProviderDataInput,
    AddExternalModelInput,
    ExternalModelProviderName,
    AttachModel,
    UpdateModelService,
    ModelData,
    JobData,
    ModelServiceData,
    ListModelsModels,
    AddHFModelInput,
    ModelFilter,
    ModelPlacementInput,
    ModelComputeConfigInput,
)
from adaptive_sdk import input_types

from .base_resource import SyncAPIResource, AsyncAPIResource, UseCaseResource

if TYPE_CHECKING:
    from adaptive_sdk.client import Adaptive, AsyncAdaptive

provider_config = {
    "open_ai": {
        "provider_data": lambda api_key, model_id: ModelProviderDataInput(
            openAI=OpenAIProviderDataInput(
                apiKey=api_key, externalModelId=OpenAIModel(model_id)
            )
        ),
    },
    "google": {
        "provider_data": lambda api_key, model_id: ModelProviderDataInput(
            google=GoogleProviderDataInput(apiKey=api_key, externalModelId=model_id)
        ),
    },
}

SupportedHFModels = Literal[
    "deepseek-ai/deepseek-coder-1.3b-base",
    "deepseek-ai/deepseek-coder-6.7b-base",
    "deepseek-ai/DeepSeek-R1-Distill-Qwen-7B",
    "deepseek-ai/DeepSeek-R1-Distill-Llama-8B",
    "tiiuae/falcon-7b",
    "tiiuae/falcon-7b-instruct",
    "tiiuae/falcon-40b",
    "tiiuae/falcon-180B",
    "BAAI/bge-multilingual-gemma2",
    "Locutusque/TinyMistral-248M",
    "mistralai/Mistral-Small-24B-Instruct-2501",
    "baffo32/decapoda-research-llama-7B-hf",
    "princeton-nlp/Sheared-LLaMA-1.3B",
    "meta-llama/Llama-3.1-8B",
    "meta-llama/Llama-3.1-8B-Instruct",
    "meta-llama/Llama-3.1-70B-Instruct",
    "meta-llama/Llama-3.3-70B-Instruct",
    "nvidia/Llama3-ChatQA-1.5-70B",
    "Qwen/Qwen2.5-0.5B",
    "Qwen/Qwen2.5-0.5B-Instruct",
    "Qwen/Qwen2.5-Coder-7B",
    "Qwen/Qwen2.5-Coder-7B-Instruct",
    "Qwen/Qwen2.5-Math-7B",
    "Qwen/Qwen2.5-Math-7B-Instruct",
    "Qwen/Qwen2.5-Coder-14B-Instruct",
    "Qwen/Qwen2.5-Coder-32B-Instruct",
    "Qwen/QwQ-32B",
    "google/gemma-3-1b-it",
    "google/gemma-3-4b-it",
    "google/gemma-3-12b-it",
    "google/gemma-3-27b-it",
    "Qwen/Qwen3-0.6B",
    "Qwen/Qwen3-1.7B",
    "Qwen/Qwen3-4B",
    "Qwen/Qwen3-8B",
    "Qwen/Qwen3-14B",
    "Qwen/Qwen3-32B",
    "01-ai/Yi-34B",
    "HuggingFaceH4/zephyr-7b-beta",
]


def is_supported_model(model_id: str):
    supported_models = get_args(SupportedHFModels)
    if model_id not in supported_models:
        supported_models_str = "\n".join(supported_models)
        raise ValueError(
            f"Model {model_id} is not supported.\n\nChoose from:\n{supported_models_str}"
        )


class Models(SyncAPIResource, UseCaseResource):  # type: ignore[misc]
    """
    Resource to interact with models.
    """

    def __init__(self, client: Adaptive) -> None:
        SyncAPIResource.__init__(self, client)
        UseCaseResource.__init__(self, client)

    def add_hf_model(
        self,
        hf_model_id: SupportedHFModels,
        output_model_name: str,
        output_model_key: str,
        hf_token: str,
        compute_pool: str | None = None,
    ) -> JobData:
        """
        Add model from the HuggingFace Model hub to Adaptive model registry.
        It will take several minutes for the model to be downloaded and converted to Adaptive format.

        Args:
            hf_model_id: The ID of the selected model repo on HuggingFace Model Hub.
            output_model_key: The key that will identify the new model in Adaptive.
            hf_token: Your HuggingFace Token, needed to validate access to gated/restricted model.
        """
        is_supported_model(hf_model_id)
        input = AddHFModelInput(
            modelId=hf_model_id,
            outputModelName=output_model_name,
            outputModelKey=output_model_key,
            hfToken=hf_token,
            computePool=compute_pool,
        )
        return self._gql_client.add_hf_model(input).import_hf_model

    def add_external(
        self,
        name: str,
        external_model_id: str,
        api_key: str,
        provider: Literal["open_ai", "google", "azure"],
        endpoint: str | None = None,
    ) -> ModelData:
        """
        Add proprietary external model to Adaptive model registry.

        Args:
            name: Adaptive name for the new model.
            external_model_id: Should match the model id publicly shared by the model provider.
            api_key: API Key for authentication against external model provider.
            provider: External proprietary model provider.
        """

        provider_data_fn: Callable[..., ModelProviderDataInput] = provider_config[provider]["provider_data"]  # type: ignore[index]
        match provider:
            case "open_ai":
                provider_data = provider_data_fn(api_key, external_model_id)
            case "google":
                provider_data = provider_data_fn(api_key, external_model_id)
            case "azure":
                if not endpoint:
                    raise ValueError(
                        "`endpoint` is required to connect Azure external model."
                    )
                provider_data = provider_data_fn(api_key, external_model_id, endpoint)
            case _:
                raise ValueError(f"Provider {provider} is not supported")

        provider_enum = ExternalModelProviderName(provider.upper())
        input = AddExternalModelInput(
            name=name, provider=provider_enum, providerData=provider_data
        )
        return self._gql_client.add_external_model(input).add_external_model

    def list(
        self, filter: input_types.ModelFilter | None = None
    ) -> Sequence[ListModelsModels]:
        """
        List all models in Adaptive model registry.
        """
        input = ModelFilter.model_validate(filter or {})
        return self._gql_client.list_models(input).models

    def get(self, model) -> ModelData | None:
        """
        Get the details for a model.

        Args:
            model: Model key.
        """
        return self._gql_client.describe_model(input=model).model

    def attach(
        self,
        model: str,
        wait: bool = False,
        make_default: bool = False,
        use_case: str | None = None,
        placement: input_types.ModelPlacementInput | None = None,
    ) -> ModelServiceData:
        """
        Attach a model to the client's use case.

        Args:
            model: Model key.
            wait: If the model is not deployed already, attaching it to the use case will automatically deploy it.
                If `True`, this call blocks until model is `Online`.
            make_default: Make the model the use case's default on attachment.
        """
        input = AttachModel(
            model=model,
            useCase=self.use_case_key(use_case),
            attached=True,
            wait=wait,
            placement=(
                ModelPlacementInput.model_validate(placement) if placement else None
            ),
        )
        result = self._gql_client.attach_model_to_use_case(input).attach_model
        if make_default:
            result = self.update(model=model, is_default=make_default)  # type: ignore[assignment]
        return result

    def detach(
        self,
        model: str,
        use_case: str | None = None,
    ) -> ModelServiceData:
        """
        Detach model from client's use case.

        Args:
            model: Model key.
        """
        return self.update(model=model, attached=False, use_case=use_case)

    def update_compute_config(
        self,
        model: str,
        compute_config: input_types.ModelComputeConfigInput,
    ) -> ModelData:
        """
        Update compute config of model.
        """
        return self._gql_client.update_model_compute_config(
            id_or_key=model,
            input=ModelComputeConfigInput.model_validate(compute_config),
        ).update_model_compute_config

    def update(
        self,
        model: str,
        is_default: bool | None = None,
        attached: bool | None = None,
        desired_online: bool | None = None,
        use_case: str | None = None,
        placement: input_types.ModelPlacementInput | None = None,
    ) -> ModelServiceData:
        """
        Update config of model attached to client's use case.

        Args:
            model: Model key.
            is_default: Change the selection of the model as default for the use case.
                `True` to promote to default, `False` to demote from default. If `None`, no changes are applied.
            attached: Whether model should be attached or detached to/from use case. If `None`, no changes are applied.
            desired_online: Turn model inference on or off for the client use case.
                This does not influence the global status of the model, it is use case-bounded.
                If `None`, no changes are applied.

        """
        input = UpdateModelService(
            useCase=self.use_case_key(use_case),
            modelService=model,
            isDefault=is_default,
            attached=attached,
            desiredOnline=desired_online,
            placement=(
                ModelPlacementInput.model_validate(placement) if placement else None
            ),
        )
        return self._gql_client.update_model(input).update_model_service

    def deploy(self, model: str, wait: bool = False) -> str:
        """
        Deploy a model, loading it to memory and making it ready for inference.

        Args:
            model: Model key.
            wait: If `True`, call block until model is in `Online` state.
        """
        return self._gql_client.deploy_model(id_or_key=model, wait=wait).deploy_model

    def terminate(self, model: str, force: bool = False) -> str:
        """
        Terminate model, removing it from memory and making it unavailable to all use cases.

        Args:
            model: Model key.
            force: If model is attached to several use cases, `force` must equal `True` in order
                for the model to be terminated.
        """
        return self._gql_client.terminate_model(
            id_or_key=model, force=force
        ).terminate_model


class AsyncModels(AsyncAPIResource, UseCaseResource):  # type: ignore[misc]
    """
    Resource to interact with models.
    """

    def __init__(self, client: AsyncAdaptive) -> None:
        AsyncAPIResource.__init__(self, client)
        UseCaseResource.__init__(self, client)

    async def add_hf_model(
        self,
        hf_model_id: SupportedHFModels,
        output_model_name: str,
        output_model_key: str,
        hf_token: str,
        compute_pool: str | None = None,
    ) -> JobData:
        """
        Add model from the HuggingFace Model hub to Adaptive model registry.
        It will take several minutes for the model to be downloaded and converted to Adaptive format.

        Args:
            hf_model_id: The ID of the selected model repo on HuggingFace Model Hub.
            output_model_key: The key that will identify the new model in Adaptive.
            hf_token: Your HuggingFace Token, needed to validate access to gated/restricted model.
        """
        is_supported_model(hf_model_id)
        input = AddHFModelInput(
            modelId=hf_model_id,
            outputModelName=output_model_name,
            outputModelKey=output_model_key,
            hfToken=hf_token,
            computePool=compute_pool,
        )
        result = await self._gql_client.add_hf_model(input)
        return result.import_hf_model

    async def add_external(
        self,
        name: str,
        external_model_id: str,
        api_key: str,
        provider: Literal["open_ai", "google", "azure"],
        endpoint: str | None = None,
    ) -> ModelData:
        """
        Add proprietary external model to Adaptive model registry.

        Args:
            name: Adaptive name for the new model.
            external_model_id: Should match the model id publicly shared by the model provider.
            api_key: API Key for authentication against external model provider.
            provider: External proprietary model provider.
        """
        provider_data_fn: Callable[..., ModelProviderDataInput] = provider_config[provider]["provider_data"]  # type: ignore[index]
        match provider:
            case "open_ai":
                provider_data = provider_data_fn(api_key, external_model_id)
            case "google":
                provider_data = provider_data_fn(api_key, external_model_id)
            case "azure":
                if not endpoint:
                    raise ValueError(
                        "`endpoint` is required to connect Azure external model."
                    )
                provider_data = provider_data_fn(api_key, external_model_id, endpoint)
            case _:
                raise ValueError(f"Provider {provider} is not supported")

        provider_enum = ExternalModelProviderName(provider.upper())
        input = AddExternalModelInput(
            name=name, provider=provider_enum, providerData=provider_data
        )
        result = await self._gql_client.add_external_model(input)
        return result.add_external_model

    async def list(
        self, filter: input_types.ModelFilter | None = None
    ) -> Sequence[ListModelsModels]:
        """
        List all models in Adaptive model registry.
        """
        input = ModelFilter.model_validate(filter or {})
        return (await self._gql_client.list_models(input)).models

    async def get(self, model) -> ModelData | None:
        """
        Get the details for a model.

        Args:
            model: Model key.
        """
        return (await self._gql_client.describe_model(input=model)).model

    async def attach(
        self,
        model: str,
        wait: bool = True,
        make_default: bool = False,
        use_case: str | None = None,
        placement: input_types.ModelPlacementInput | None = None,
    ) -> ModelServiceData:
        """
        Attach a model to the client's use case.

        Args:
            model: Model key.
            wait: If the model is not deployed already, attaching it to the use case will automatically deploy it.
                If `True`, this call blocks until model is `Online`.
            make_default: Make the model the use case's default on attachment.
        """
        input = AttachModel(
            model=model,
            useCase=self.use_case_key(use_case),
            attached=True,
            wait=wait,
            placement=(
                ModelPlacementInput.model_validate(placement) if placement else None
            ),
        )
        result = await self._gql_client.attach_model_to_use_case(input)
        result = result.attach_model
        if make_default:
            result = await self.update(model=model, is_default=make_default)  # type: ignore[assignment]
        return result

    async def detach(
        self,
        model: str,
        use_case: str | None = None,
    ) -> ModelServiceData:
        """
        Detach model from client's use case.

        Args:
            model: Model key.
        """
        return await self.update(model=model, attached=False, use_case=use_case)

    async def update_compute_config(
        self,
        model: str,
        compute_config: input_types.ModelComputeConfigInput,
    ) -> ModelData:
        """
        Update compute config of model.
        """
        return (
            await self._gql_client.update_model_compute_config(
                id_or_key=model,
                input=ModelComputeConfigInput.model_validate(compute_config),
            )
        ).update_model_compute_config

    async def update(
        self,
        model: str,
        is_default: bool | None = None,
        attached: bool | None = None,
        desired_online: bool | None = None,
        use_case: str | None = None,
        placement: input_types.ModelPlacementInput | None = None,
    ) -> ModelServiceData:
        """
        Update config of model attached to client's use case.

        Args:
            model: Model key.
            is_default: Change the selection of the model as default for the use case.
                `True` to promote to default, `False` to demote from default. If `None`, no changes are applied.
            attached: Whether model should be attached or detached to/from use case. If `None`, no changes are applied.
            desired_online: Turn model inference on or off for the client use case.
                This does not influence the global status of the model, it is use case-bounded.
                If `None`, no changes are applied.

        """
        input = UpdateModelService(
            useCase=self.use_case_key(use_case),
            modelService=model,
            isDefault=is_default,
            attached=attached,
            desiredOnline=desired_online,
            placement=(
                ModelPlacementInput.model_validate(placement) if placement else None
            ),
        )
        result = await self._gql_client.update_model(input)
        return result.update_model_service

    async def deploy(self, model: str, wait: bool = False) -> str:
        """
        Deploy a model, loading it to memory and making it ready for inference.

        Args:
            model: Model key.
            wait: If `True`, call block until model is in `Online` state.
        """
        return (
            await self._gql_client.deploy_model(id_or_key=model, wait=wait)
        ).deploy_model

    async def terminate(self, model: str, force: bool = False) -> str:
        """
        Terminate model, removing it from memory and making it unavailable to all use cases.

        Args:
            model: Model key.
            force: If model is attached to several use cases, `force` must equal `True` in order
                for the model to be terminated.
        """
        return (
            await self._gql_client.terminate_model(id_or_key=model, force=force)
        ).terminate_model



# type: ignore

from __future__ import annotations
from typing import TYPE_CHECKING
from pydantic import BaseModel

from adaptive_sdk.graphql_client import ResizePartitionInput, HarmonyStatus

from .base_resource import SyncAPIResource, AsyncAPIResource, UseCaseResource

if TYPE_CHECKING:
    from adaptive_sdk.client import Adaptive, AsyncAdaptive


class ResizeResult(BaseModel):
    harmony_group_key: str
    success: bool
    error: str | None = None


class ComputePools(SyncAPIResource, UseCaseResource):
    """
    Resource to interact with compute pools.
    """

    def __init__(self, client: Adaptive) -> None:
        SyncAPIResource.__init__(self, client)
        UseCaseResource.__init__(self, client)

    def list(self):
        return self._gql_client.list_compute_pools().compute_pools

    def resize_inference_partition(self, compute_pool_key: str, size: int) -> list[ResizeResult]:
        """
        Resize the inference partitions of all harmony groups in a compute pool.
        """
        cps = self.list()
        found_cp = False
        for cp in cps.compute_pools:
            if cp.key == compute_pool_key:
                selected_cp = cp
                found_cp = True
                break
        if not found_cp:
            raise ValueError(f"Compute pool with key {compute_pool_key} not found")

        resize_results: list[ResizeResult] = []
        for hg in selected_cp.harmony_groups:
            if hg.status == HarmonyStatus.ONLINE:
                input = ResizePartitionInput(harmonyGroup=hg.key, size=size)
                try:
                    _ = self._gql_client.resize_inference_partition(input)
                    resize_results.append(ResizeResult(harmony_group_key=hg.key, success=True))
                except Exception as e:
                    resize_results.append(ResizeResult(harmony_group_key=hg.key, success=False, error_message=str(e)))

        return resize_results


class AsyncComputePools(AsyncAPIResource, UseCaseResource):
    """
    Resource to interact with compute pools.
    """

    def __init__(self, client: AsyncAdaptive) -> None:
        AsyncAPIResource.__init__(self, client)
        UseCaseResource.__init__(self, client)

    async def list(self):
        return (await self._gql_client.list_compute_pools()).compute_pools

    async def resize_inference_partition(self, compute_pool_key: str, size: int) -> list[ResizeResult]:
        """
        Resize the inference partitions of all harmony groups in a compute pool.
        """
        cps = await self.list()
        found_cp = False
        for cp in cps.compute_pools:
            if cp.key == compute_pool_key:
                selected_cp = cp
                found_cp = True
                break
        if not found_cp:
            raise ValueError(f"Compute pool with key {compute_pool_key} not found")

        resize_results: list[ResizeResult] = []
        for hg in selected_cp.harmony_groups:
            if hg.status == HarmonyStatus.ONLINE:
                input = ResizePartitionInput(harmonyGroup=hg.key, size=size)
                try:
                    _ = await self._gql_client.resize_inference_partition(input)
                    resize_results.append(ResizeResult(harmony_group_key=hg.key, success=True))
                except Exception as e:
                    resize_results.append(ResizeResult(harmony_group_key=hg.key, success=False, error_message=str(e)))

        return resize_results



from typing import Any, Optional
from pydantic import Field
from .base_model import BaseModel
from .enums import SessionStatus

class CreateDatasetFromMultipartUpload(BaseModel):
    """@public"""
    create_dataset_from_multipart_upload: 'CreateDatasetFromMultipartUploadCreateDatasetFromMultipartUpload' = Field(alias='createDatasetFromMultipartUpload')

class CreateDatasetFromMultipartUploadCreateDatasetFromMultipartUpload(BaseModel):
    """@public"""
    dataset_id: Any = Field(alias='datasetId')
    status: SessionStatus
    total_parts: int = Field(alias='totalParts')
    processed_parts: int = Field(alias='processedParts')
    progress: float
    error: Optional[str]
CreateDatasetFromMultipartUpload.model_rebuild()


from typing import Any, List
from pydantic import Field
from .base_model import BaseModel
from .fragments import UserData

class UpdateUser(BaseModel):
    """@public"""
    set_team_member: 'UpdateUserSetTeamMember' = Field(alias='setTeamMember')

class UpdateUserSetTeamMember(BaseModel):
    """@public"""
    user: 'UpdateUserSetTeamMemberUser'
    team: 'UpdateUserSetTeamMemberTeam'
    role: 'UpdateUserSetTeamMemberRole'

class UpdateUserSetTeamMemberUser(UserData):
    """@public"""
    pass

class UpdateUserSetTeamMemberTeam(BaseModel):
    """@public"""
    id: Any
    key: str
    name: str
    created_at: int = Field(alias='createdAt')

class UpdateUserSetTeamMemberRole(BaseModel):
    """@public"""
    id: Any
    key: str
    name: str
    created_at: int = Field(alias='createdAt')
    permissions: List[str]
UpdateUser.model_rebuild()
UpdateUserSetTeamMember.model_rebuild()


"""
This file serves the purpose of automatic SDK docs generation.
All output rest types are included in __all__ so they are exported to the reference docs.
"""
from .rest_types import AddInteractionsResponse, ChatChoice, ChatChoiceMessage, ChatResponse, ChatResponseChunk, ComparisonOutput, Delta, EmbeddingResponse, EmbeddingsResponseList, FeedbackOutput, GenerateChoice, Usage
__all__ = ['AddInteractionsResponse', 'ChatResponse', 'ChatChoice', 'ChatChoiceMessage', 'ChatResponseChunk', 'ComparisonOutput', 'Delta', 'EmbeddingResponse', 'EmbeddingsResponseList', 'FeedbackOutput', 'GenerateChoice', 'Usage']


from __future__ import annotations
import json
from loguru import logger
from uuid import UUID
from typing import (
    Dict,
    List,
    AsyncGenerator,
    Generator,
    Literal,
    overload,
    TYPE_CHECKING,
)
from typing_extensions import override
from adaptive_sdk import input_types
from adaptive_sdk.error_handling import rest_error_handler
from adaptive_sdk.rest import rest_types
from adaptive_sdk.utils import convert_optional_UUID, get_full_model_path

from .base_resource import SyncAPIResource, AsyncAPIResource, UseCaseResource

if TYPE_CHECKING:
    from adaptive_sdk.client import Adaptive, AsyncAdaptive

ROUTE = "/chat/completions"


class Chat(SyncAPIResource, UseCaseResource):  # type: ignore[misc]
    @override
    def __init__(self, client: Adaptive) -> None:
        SyncAPIResource.__init__(self, client)
        UseCaseResource.__init__(self, client)

    @overload
    def create(
        self,
        messages: List[input_types.ChatMessage],
        stream: Literal[False] | None = None,
        model: str | None = None,
        stop: List[str] | None = None,
        max_tokens: int | None = None,
        temperature: float | None = None,
        top_p: float | None = None,
        stream_include_usage: bool | None = None,
        session_id: str | UUID | None = None,
        use_case: str | None = None,
        user: str | UUID | None = None,
        ab_campaign: str | None = None,
        n: int | None = None,
        labels: Dict[str, str] | None = None,
    ) -> rest_types.ChatResponse: ...

    @overload
    def create(
        self,
        messages: List[input_types.ChatMessage],
        stream: Literal[True],
        model: str | None = None,
        stop: List[str] | None = None,
        max_tokens: int | None = None,
        temperature: float | None = None,
        top_p: float | None = None,
        stream_include_usage: bool | None = None,
        session_id: str | UUID | None = None,
        use_case: str | None = None,
        user: str | UUID | None = None,
        ab_campaign: str | None = None,
        n: int | None = None,
        labels: Dict[str, str] | None = None,
    ) -> Generator[rest_types.ChatResponseChunk, None, None]: ...

    def create(
        self,
        messages: List[input_types.ChatMessage],
        stream: bool | None = None,
        model: str | None = None,
        stop: List[str] | None = None,
        max_tokens: int | None = None,
        temperature: float | None = None,
        top_p: float | None = None,
        stream_include_usage: bool | None = None,
        session_id: str | UUID | None = None,
        use_case: str | None = None,
        user: str | UUID | None = None,
        ab_campaign: str | None = None,
        n: int | None = None,
        labels: Dict[str, str] | None = None,
    ) -> rest_types.ChatResponse | Generator[rest_types.ChatResponseChunk, None, None]:
        """
        Create a chat completion.

        Args:
            messages: Input messages, each dict with keys `role` and `content`.
            stream: If `True`, partial message deltas will be returned. If stream is over, chunk.choices will be None.
            model: Target model key for inference. If `None`, the requests will be routed to the use case's default model.
            stop: Sequences or where the API will stop generating further tokens.
            max_tokens: Maximum # of tokens allowed to generate.
            temperature: Sampling temperature.
            top_p: Threshold for top-p sampling.
            stream_include_usage: If set, an additional chunk will be streamed with the token usage statistics for
                the entire request.
            user: ID of user making request. If not `None`, will be logged as metadata for the request.
            ab_campaign: AB test key. If set, request will be guaranteed to count towards AB test results,
                no matter the configured `traffic_split`.
            n: Number of chat completions to generate for each input messages.
            labels: Key-value pairs of interaction labels.

        Examples:
        ```
        # streaming chat request
        stream_response = client.chat.create(
            model="model_key", messages=[{"role": "user", "content": "Hello from SDK"}], stream=True
        )

        print("Streaming response: ", end="", flush=True)
        for chunk in stream_response:
            if chunk.choices:
                content = chunk.choices[0].delta.content
                print(content, end="", flush=True)
        ```
        """
        # TODO: figure out what to do with possible completion_id in messages
        input_messages = [rest_types.ChatMessage(**m) for m in messages]
        input = rest_types.ChatInput(
            messages=input_messages,
            model=get_full_model_path(self.use_case_key(use_case), model),
            stream=stream,
            stop=stop,
            max_tokens=max_tokens,
            temperature=temperature,
            top_p=top_p,
            stream_options=rest_types.StreamOptions(include_usage=stream_include_usage),
            session_id=convert_optional_UUID(session_id),
            user=convert_optional_UUID(user),
            ab_campaign=ab_campaign,
            n=n,
            labels=labels,
        )
        if input.stream:
            return self._stream(input)
        r = self._rest_client.post(ROUTE, json=input.model_dump(exclude_none=True))
        rest_error_handler(r)
        return rest_types.ChatResponse.model_validate(r.json())

    def _stream(
        self, input: rest_types.ChatInput
    ) -> Generator[rest_types.ChatResponseChunk, None, None]:
        import httpx_sse

        with httpx_sse.connect_sse(
            self._rest_client, "POST", ROUTE, json=input.model_dump(exclude_none=True)
        ) as event_source:
            for sse in event_source.iter_sse():
                if sse.data == "[DONE]":
                    break
                try:
                    chunk_content = json.loads(sse.data)
                except Exception as e:
                    logger.warning(f"Error with json in chunk: {sse.data}")
                    continue
                yield rest_types.ChatResponseChunk.model_validate(chunk_content)


class AsyncChat(AsyncAPIResource, UseCaseResource):  # type: ignore[misc]
    def __init__(self, client: AsyncAdaptive) -> None:
        AsyncAPIResource.__init__(self, client)
        UseCaseResource.__init__(self, client)

    @overload
    async def create(  # type: ignore[empty-body]
        self,
        messages: List[input_types.ChatMessage],
        stream: Literal[False] | None = None,
        model: str | None = None,
        stop: List[str] | None = None,
        max_tokens: int | None = None,
        temperature: float | None = None,
        top_p: float | None = None,
        stream_include_usage: bool | None = None,
        session_id: str | UUID | None = None,
        use_case: str | None = None,
        user: str | UUID | None = None,
        ab_campaign: str | None = None,
        n: int | None = None,
        labels: Dict[str, str] | None = None,
    ) -> rest_types.ChatResponse: ...

    @overload  # type: ignore[no-redef, misc]
    async def create(
        self,
        messages: List[input_types.ChatMessage],
        stream: Literal[True],
        model: str | None = None,
        stop: List[str] | None = None,
        max_tokens: int | None = None,
        temperature: float | None = None,
        top_p: float | None = None,
        stream_include_usage: bool | None = None,
        session_id: str | UUID | None = None,
        use_case: str | None = None,
        user: str | UUID | None = None,
        ab_campaign: str | None = None,
        n: int | None = None,
        labels: Dict[str, str] | None = None,
    ) -> AsyncGenerator[rest_types.ChatResponseChunk, None]: ...

    async def create(  # type: ignore
        self,
        messages: List[input_types.ChatMessage],
        model: str | None = None,
        stream: bool | None = None,
        stop: List[str] | None = None,
        max_tokens: int | None = None,
        temperature: float | None = None,
        top_p: float | None = None,
        stream_include_usage: bool | None = None,
        session_id: str | UUID | None = None,
        use_case: str | None = None,
        user: str | UUID | None = None,
        ab_campaign: str | None = None,
        n: int | None = None,
        labels: Dict[str, str] | None = None,
    ) -> rest_types.ChatResponse | AsyncGenerator[rest_types.ChatResponseChunk, None]:
        """
        Create a chat completion.

        Args:
            messages: Input messages, each dict with keys `role` and `content`.
            stream: If `True`, partial message deltas will be returned.
            model: Target model key for inference. If `None`, the requests will be routed to the use case's default model.
            stop: Sequences or where the API will stop generating further tokens.
            max_tokens: Maximum # of tokens allowed to generate.
            temperature: Sampling temperature.
            top_p: Threshold for top-p sampling.
            stream_include_usage: If set, an additional chunk will be streamed with the token usaage statistics for
                the entire request.
            user: ID of user making request. If not `None`, will be logged as metadata for the request.
            ab_campaign: AB test key. If set, request will be guaranteed to count towards AB test results,
                no matter the configured `traffic_split`.
            n: Number of chat completions to generate for each input messages.
            labels: Key-value pairs of interaction labels.

        Examples:
        ```
        # async streaming chat request
        async def async_chat_stream():
            stream_response = aclient.chat.create(
                model="model_key", messages=[{"role": "user", "content": "Hello from SDK"}], stream=True
            )

            print("Async chat streaming response: ", end="", flush=True)
            async for chunk in await stream_response:
                if chunk.choices:
                    content = chunk.choices[0].delta.content
                    print(content, end="", flush=True)
        ```
        """
        input_messages = [rest_types.ChatMessage(**m) for m in messages]
        input = rest_types.ChatInput(
            messages=input_messages,
            model=get_full_model_path(self.use_case_key(use_case), model),
            stream=stream,
            stop=stop,
            max_tokens=max_tokens,
            temperature=temperature,
            top_p=top_p,
            stream_options=rest_types.StreamOptions(include_usage=stream_include_usage),
            session_id=convert_optional_UUID(session_id),
            user=convert_optional_UUID(user),
            ab_campaign=ab_campaign,
            n=n,
            labels=labels,
        )
        if input.stream:
            return self._stream(input)
        r = await self._rest_client.post(
            ROUTE, json=input.model_dump(exclude_none=True)
        )
        rest_error_handler(r)
        return rest_types.ChatResponse.model_validate(r.json())

    async def _stream(
        self, input: rest_types.ChatInput
    ) -> AsyncGenerator[rest_types.ChatResponseChunk, None]:
        import httpx_sse

        async with httpx_sse.aconnect_sse(
            self._rest_client, "POST", ROUTE, json=input.model_dump(exclude_none=True)
        ) as event_source:
            async for sse in event_source.aiter_sse():
                if sse.data == "[DONE]":
                    break
                try:
                    chunk_content = json.loads(sse.data)
                except Exception as e:
                    logger.warning(f"Error with json in chunk: {sse.data}")
                    continue
                yield rest_types.ChatResponseChunk.model_validate(chunk_content)



from pydantic import Field
from .base_model import BaseModel
from .fragments import ModelData

class UpdateModelComputeConfig(BaseModel):
    """@public"""
    update_model_compute_config: 'UpdateModelComputeConfigUpdateModelComputeConfig' = Field(alias='updateModelComputeConfig')

class UpdateModelComputeConfigUpdateModelComputeConfig(ModelData):
    """@public"""
    pass
UpdateModelComputeConfig.model_rebuild()


from __future__ import annotations
import json
import math
import os
import time
from pathlib import Path
from typing import List, TYPE_CHECKING

from adaptive_sdk.graphql_client import (
    DatasetCreate,
    Upload,
    ListDatasetsDatasets,
    DatasetData,
    DatasetCreateFromMultipartUpload,
    DatasetUploadProcessingStatusInput,
    SessionStatus,
)

from adaptive_sdk.rest import rest_types
from adaptive_sdk.error_handling import rest_error_handler

from .base_resource import SyncAPIResource, AsyncAPIResource, UseCaseResource

if TYPE_CHECKING:
    from adaptive_sdk.client import Adaptive, AsyncAdaptive

MIN_CHUNK_SIZE_BYTES = 5 * 1024 * 1024  # 5MB
MAX_CHUNK_SIZE_BYTES = 100 * 1024 * 1024  # 100MB
MAX_PARTS_COUNT = 10000
INIT_CHUNKED_UPLOAD_ROUTE = "/upload/init"
UPLOAD_PART_ROUTE = "/upload/part"
ABORT_CHUNKED_UPLOAD_ROUTE = "/upload/abort"


def _calculate_upload_parts(file_size: int) -> tuple[int, int]:
    if file_size < MIN_CHUNK_SIZE_BYTES:
        raise ValueError(f"File size ({file_size:,} bytes) is too small for chunked upload")

    # Select appropriate chunk size based on file size
    SIZE_500MB = 500 * 1024 * 1024
    SIZE_10GB = 10 * 1024 * 1024 * 1024
    SIZE_50GB = 50 * 1024 * 1024 * 1024

    if file_size < SIZE_500MB:
        chunk_size = 5 * 1024 * 1024  # 5MB
    elif file_size < SIZE_10GB:
        chunk_size = 10 * 1024 * 1024  # 10MB
    elif file_size < SIZE_50GB:
        chunk_size = 50 * 1024 * 1024  # 50MB
    else:
        chunk_size = 100 * 1024 * 1024  # 100MB

    total_parts = math.ceil(file_size / chunk_size)

    if total_parts > MAX_PARTS_COUNT:
        # Calculate minimum chunk size needed to stay under max parts
        chunk_size = math.ceil(file_size / MAX_PARTS_COUNT)

        # Check if we exceed max chunk size
        if chunk_size > MAX_CHUNK_SIZE_BYTES:
            max_file_size = MAX_CHUNK_SIZE_BYTES * MAX_PARTS_COUNT
            raise ValueError(
                f"File size ({file_size:,} bytes) exceeds maximum uploadable size "
                f"({max_file_size:,} bytes = {MAX_PARTS_COUNT} parts * {MAX_CHUNK_SIZE_BYTES:,} bytes)"
            )

        total_parts = math.ceil(file_size / chunk_size)

    return total_parts, chunk_size


class Datasets(SyncAPIResource, UseCaseResource):  # type: ignore[misc]
    """
    Resource to interact with file datasets.
    """

    def __init__(self, client: Adaptive) -> None:
        SyncAPIResource.__init__(self, client)
        UseCaseResource.__init__(self, client)

    def upload(
        self,
        file_path: str,
        dataset_key: str,
        name: str | None = None,
        use_case: str | None = None,
    ) -> DatasetData:
        """
        Upload a dataset from a file. File must be jsonl, where each line should match supported structure.

        Args:
            file_path: Path to jsonl file.
            dataset_key: New dataset key.
            name: Optional name to render in UI; if `None`, defaults to same as `dataset_key`.

        """
        file_size = os.path.getsize(file_path)

        if file_size > MIN_CHUNK_SIZE_BYTES:
            return self._chunked_upload(file_path, dataset_key, name, use_case)
        else:
            # Use standard GraphQL upload for smaller files
            input = DatasetCreate(
                useCase=self.use_case_key(use_case),
                name=name if name else dataset_key,
                key=dataset_key,
            )
            filename = Path(file_path).stem
            with open(file_path, "rb") as f:
                file_upload = Upload(filename=filename, content=f, content_type="application/jsonl")
                return self._gql_client.load_dataset(input=input, file=file_upload).create_dataset

    def _chunked_upload(
        self,
        file_path: str,
        dataset_key: str,
        name: str | None = None,
        use_case: str | None = None,
    ) -> DatasetData:
        """Upload large files using chunked upload via REST API."""
        file_size = os.path.getsize(file_path)
        total_parts, chunk_size = _calculate_upload_parts(file_size)

        init_request = rest_types.InitChunkedUploadRequest(
            content_type="application/jsonl",
            metadata=None,
            total_parts_count=total_parts,
        )
        response = self._rest_client.post(INIT_CHUNKED_UPLOAD_ROUTE, json=init_request.model_dump())
        rest_error_handler(response)
        init_response = rest_types.InitChunkedUploadResponse.model_validate(response.json())
        session_id = init_response.session_id

        try:
            with open(file_path, "rb") as f:
                for part_number in range(1, total_parts + 1):
                    chunk_data = f.read(chunk_size)

                    response = self._rest_client.post(
                        UPLOAD_PART_ROUTE,
                        params={"session_id": session_id, "part_number": part_number},
                        content=chunk_data,
                        headers={"Content-Type": "application/octet-stream"},
                    )
                    rest_error_handler(response)

            input = DatasetCreateFromMultipartUpload(
                useCase=self.use_case_key(use_case),
                name=name if name else dataset_key,
                key=dataset_key,
                uploadSessionId=session_id,
            )
            create_dataset_result = self._gql_client.create_dataset_from_multipart_upload(
                input=input
            ).create_dataset_from_multipart_upload

            upload_done = False
            while not upload_done:
                check_progress_result = self._gql_client.dataset_upload_processing_status(
                    input=DatasetUploadProcessingStatusInput(
                        useCase=self.use_case_key(use_case), datasetId=create_dataset_result.dataset_id
                    )
                ).dataset_upload_processing_status
                if check_progress_result.status == SessionStatus.DONE:
                    upload_done = True
                elif check_progress_result.status == SessionStatus.ERROR:
                    raise Exception(f"Upload failed: {check_progress_result.error}")
                else:
                    time.sleep(2)

            dataset_data = self.get(create_dataset_result.dataset_id, use_case=self.use_case_key(use_case))
            assert dataset_data is not None

            return dataset_data

        except Exception:
            try:
                abort_request = rest_types.AbortChunkedUploadRequest(session_id=session_id)
                self._rest_client.delete(
                    ABORT_CHUNKED_UPLOAD_ROUTE,
                    content=json.dumps(abort_request.model_dump()),  # type: ignore[call-arg]
                    headers={"Content-Type": "application/json"},
                )
            except Exception:
                pass
            raise

    def list(
        self,
        use_case: str | None = None,
    ) -> List[ListDatasetsDatasets]:
        """
        List previously uploaded datasets.
        """
        return self._gql_client.list_datasets(self.use_case_key(use_case)).datasets

    def get(self, key: str, use_case: str | None = None) -> DatasetData | None:
        """
        Get details for dataset.

        Args:
            key: Dataset key.
        """
        return self._gql_client.describe_dataset(key, self.use_case_key(use_case)).dataset

    def delete(self, key: str, use_case: str | None = None) -> bool:
        """Delete dataset."""
        return self._gql_client.delete_dataset(id_or_key=key, use_case=self.use_case_key(use_case)).delete_dataset


class AsyncDatasets(AsyncAPIResource, UseCaseResource):  # type: ignore[misc]
    def __init__(self, client: AsyncAdaptive) -> None:
        AsyncAPIResource.__init__(self, client)
        UseCaseResource.__init__(self, client)

    async def upload(
        self,
        file_path: str,
        dataset_key: str,
        name: str | None = None,
        use_case: str | None = None,
    ) -> DatasetData:
        """
        Upload a dataset from a file. File must be jsonl, where each line should match structure in example below.

        Args:
            file_path: Path to jsonl file.
            dataset_key: New dataset key.
            name: Optional name to render in UI; if `None`, defaults to same as `dataset_key`.

        Example:
        ```
        {"messages": [{"role": "system", "content": "<optional system prompt>"}, {"role": "user", "content": "<user content>"}, {"role": "assistant", "content": "<assistant answer>"}], "completion": "hey"}
        ```
        """
        file_size = os.path.getsize(file_path)

        # Use chunked upload for files larger than MIN_CHUNK_SIZE_BYTES
        if file_size > MIN_CHUNK_SIZE_BYTES:
            return await self._chunked_upload(file_path, dataset_key, name, use_case)
        else:
            # Use standard GraphQL upload for smaller files
            input = DatasetCreate(
                useCase=self.use_case_key(use_case),
                name=name if name else dataset_key,
                key=dataset_key,
            )
            filename = Path(file_path).stem
            with open(file_path, "rb") as f:
                file_upload = Upload(filename=filename, content=f, content_type="application/jsonl")
                upload_result = await self._gql_client.load_dataset(input=input, file=file_upload)
                return upload_result.create_dataset

    async def _chunked_upload(
        self,
        file_path: str,
        dataset_key: str,
        name: str | None = None,
        use_case: str | None = None,
    ) -> DatasetData:
        """Upload large files using chunked upload via REST API."""
        file_size = os.path.getsize(file_path)
        total_parts, chunk_size = _calculate_upload_parts(file_size)

        init_request = rest_types.InitChunkedUploadRequest(
            content_type="application/jsonl",
            metadata=None,
            total_parts_count=total_parts,
        )
        response = await self._rest_client.post(INIT_CHUNKED_UPLOAD_ROUTE, json=init_request.model_dump())
        rest_error_handler(response)
        init_response = rest_types.InitChunkedUploadResponse.model_validate(response.json())
        session_id = init_response.session_id

        try:
            with open(file_path, "rb") as f:
                for part_number in range(1, total_parts + 1):
                    chunk_data = f.read(chunk_size)

                    response = await self._rest_client.post(
                        UPLOAD_PART_ROUTE,
                        params={"session_id": session_id, "part_number": part_number},
                        content=chunk_data,
                        headers={"Content-Type": "application/octet-stream"},
                    )
                    rest_error_handler(response)

            input = DatasetCreateFromMultipartUpload(
                useCase=self.use_case_key(use_case),
                name=name if name else dataset_key,
                key=dataset_key,
                uploadSessionId=session_id,
            )
            create_dataset_result = (
                await self._gql_client.create_dataset_from_multipart_upload(input=input)
            ).create_dataset_from_multipart_upload

            upload_done = False
            while not upload_done:
                check_progress_result = (
                    await self._gql_client.dataset_upload_processing_status(
                        input=DatasetUploadProcessingStatusInput(
                            useCase=self.use_case_key(use_case), datasetId=create_dataset_result.dataset_id
                        )
                    )
                ).dataset_upload_processing_status
                if check_progress_result.status == SessionStatus.DONE:
                    upload_done = True
                elif check_progress_result.status == SessionStatus.ERROR:
                    raise Exception(f"Upload failed: {check_progress_result.error}")
                else:
                    time.sleep(2)

            dataset_data = await self.get(create_dataset_result.dataset_id, use_case=self.use_case_key(use_case))
            assert dataset_data is not None

            return dataset_data

        except Exception:
            try:
                abort_request = rest_types.AbortChunkedUploadRequest(session_id=session_id)
                _ = await self._rest_client.delete(
                    ABORT_CHUNKED_UPLOAD_ROUTE,
                    content=json.dumps(abort_request.model_dump()),  # type: ignore[call-arg]
                    headers={"Content-Type": "application/json"},
                )
            except Exception:
                pass
            raise

    async def list(
        self,
        use_case: str | None = None,
    ) -> List[ListDatasetsDatasets]:
        """
        List previously uploaded datasets.
        """
        results = await self._gql_client.list_datasets(self.use_case_key(use_case))
        return results.datasets

    async def get(self, key: str, use_case: str | None = None) -> DatasetData | None:
        """
        Get details for dataset.

        Args:
            key: Dataset key.
        """
        result = await self._gql_client.describe_dataset(key, self.use_case_key(use_case))
        return result.dataset

    async def delete(self, key: str, use_case: str | None = None) -> bool:
        """Delete dataset."""
        return (
            await self._gql_client.delete_dataset(id_or_key=key, use_case=self.use_case_key(use_case))
        ).delete_dataset



from typing import List
from .base_model import BaseModel
from .fragments import MetricDataAdmin

class ListMetrics(BaseModel):
    """@public"""
    metrics: List['ListMetricsMetrics']

class ListMetricsMetrics(MetricDataAdmin):
    """@public"""
    pass
ListMetrics.model_rebuild()


from typing import Optional
from .base_model import BaseModel
from .fragments import MetricDataAdmin

class DescribeMetricAdmin(BaseModel):
    """@public"""
    metric: Optional['DescribeMetricAdminMetric']

class DescribeMetricAdminMetric(MetricDataAdmin):
    """@public"""
    pass
DescribeMetricAdmin.model_rebuild()


from typing import List
from pydantic import Field
from .base_model import BaseModel
from .fragments import HarmonyGroupData

class ListHarmonyGroups(BaseModel):
    """@public"""
    harmony_groups: List['ListHarmonyGroupsHarmonyGroups'] = Field(alias='harmonyGroups')

class ListHarmonyGroupsHarmonyGroups(HarmonyGroupData):
    """@public"""
    pass
ListHarmonyGroups.model_rebuild()


from pydantic import Field
from .base_model import BaseModel
from .fragments import JobData

class AddHFModel(BaseModel):
    """@public"""
    import_hf_model: 'AddHFModelImportHfModel' = Field(alias='importHfModel')

class AddHFModelImportHfModel(JobData):
    """@public"""
    pass
AddHFModel.model_rebuild()


from abc import ABC, abstractmethod
import os
from enum import Enum
from typing import Optional, Dict
import httpx
from urllib.parse import urlparse

from .graphql_client import GQLClient, AsyncGQLClient
from .error_handling import graphql_multi_error_handler


def get_version():
    import importlib

    return importlib.import_module("adaptive_sdk").__version__


class Routes(Enum):
    GQL = "/api/graphql"
    REST = "/api/v1"


def _get_api_key(api_key: Optional[str] = None) -> str:
    if api_key:
        return api_key

    env_api_key = os.environ.get("ADAPTIVE_API_KEY")
    if env_api_key:
        return env_api_key

    raise ValueError("API key not found. Please provide an API KEY or set ADAPTIVE_API_KEY environment variable.")


def is_valid_url(url):
    parsed_url = urlparse(url)
    return bool(parsed_url.scheme) and bool(parsed_url.netloc)


class BaseSyncClient:
    def __init__(
        self,
        base_url: str,
        api_key: Optional[str] = None,
        default_headers: Dict[str, str] | None = None,
        timeout_secs: float | None = 90.0,
    ):
        """
        Construct a new synchronous client instance.

        Args:
            base_url: The base URL for the API
            api_key: API key for authentication (or set ADAPTIVE_API_KEY env var)
            default_headers: Additional headers to include in requests
            timeout_secs: Timeout in seconds for requests (None for no timeout)
        """
        if not base_url:
            raise ValueError("base_url must be provided")
        else:
            if is_valid_url(base_url):
                base_url = base_url.rstrip("/")
            else:
                raise ValueError(
                    f"Provided base_url {base_url} is invalid. It should be a well-formed url like https://api-adaptive-ml.com"
                )

        self.api_key = _get_api_key(api_key)
        self.base_url = base_url
        custom_headers = (default_headers or {}).copy()
        headers = {
            **custom_headers,
            "Authorization": f"Bearer {self.api_key}",
            "User-Agent": f"adaptive_sdk/{get_version()}",
        }
        self._gql_client = GQLClient(base_url + Routes.GQL.value, headers=headers)
        self._gql_client.get_data = graphql_multi_error_handler(self._gql_client.get_data)  # type: ignore[method-assign]

        timeout = httpx.Timeout(timeout=timeout_secs) if timeout_secs is not None else None

        self._gql_client.http_client.timeout = timeout
        self._rest_client = httpx.Client(headers=headers, base_url=base_url + Routes.REST.value, timeout=timeout)

    def close(self) -> None:
        self._rest_client.close()
        self._gql_client.http_client.close()


class BaseAsyncClient:
    def __init__(
        self,
        base_url: str,
        api_key: Optional[str] = None,
        default_headers: Dict[str, str] | None = None,
        timeout_secs: float | None = 90.0,
    ):
        """
        Construct a new asynchronous client instance.

        Args:
            base_url: The base URL for the API
            api_key: API key for authentication (or set ADAPTIVE_API_KEY env var)
            default_headers: Additional headers to include in requests
            timeout_secs: Timeout in seconds for requests (None for no timeout)
        """
        if not base_url:
            raise ValueError("base_url must be provided")
        else:
            if is_valid_url(base_url):
                base_url = base_url.rstrip("/")
            else:
                raise ValueError(
                    f"Provided base_url {base_url} is invalid. It should be a well-formed url like https://api.adaptive-ml.com"
                )

        self.api_key = _get_api_key(api_key)
        self.base_url = base_url
        custom_headers = (default_headers or {}).copy()
        headers = {
            **custom_headers,
            "Authorization": f"Bearer {self.api_key}",
            "User-Agent": f"adaptive_sdk/{get_version()}",
        }

        self._gql_client = AsyncGQLClient(base_url + Routes.GQL.value, headers=headers)
        self._gql_client.get_data = graphql_multi_error_handler(self._gql_client.get_data)  # type: ignore[method-assign]

        timeout = httpx.Timeout(timeout=timeout_secs) if timeout_secs is not None else None
        self._gql_client.http_client.timeout = timeout
        self._rest_client = httpx.AsyncClient(headers=headers, base_url=base_url + Routes.REST.value, timeout=timeout)

    async def close(self) -> None:
        await self._rest_client.aclose()
        await self._gql_client.http_client.aclose()


class UseCaseClient(ABC):
    @property
    @abstractmethod
    def default_use_case(self) -> str | None:
        """Get the current default use case key."""
        pass

    @abstractmethod
    def set_default_use_case(self, use_case: str) -> None:
        """Set the default use case key."""
        pass



from typing import Any, List, Optional
from pydantic import Field
from .base_model import BaseModel
from .enums import AbcampaignStatus, CompletionGroupBy, CompletionSource, DatasetKind, DatasetSource, DateBucketUnit, ExternalModelProviderName, FeedbackType, GraderTypeEnum, JobArtifactKind, JobKind, JobStatus, MetricAggregation, MetricKind, MetricScoringType, ModelCapabilityFilter, ModelOnline, OpenAIModel, PrebuiltCriteriaKey, Protocol, SelectionTypeInput, SortDirection, TimeseriesInterval, UnitPosition

class AbCampaignFilter(BaseModel):
    """@private"""
    active: Optional[bool] = None
    status: Optional[AbcampaignStatus] = None
    use_case: Optional[str] = Field(alias='useCase', default=None)

class AbcampaignCreate(BaseModel):
    """@private"""
    key: str
    name: Optional[str] = None
    metric: str
    use_case: str = Field(alias='useCase')
    model_services: List[str] = Field(alias='modelServices')
    auto_deploy: bool = Field(alias='autoDeploy')
    traffic_split: float = Field(alias='trafficSplit')
    feedback_type: FeedbackType = Field(alias='feedbackType', default=FeedbackType.DIRECT)

class AddExternalModelInput(BaseModel):
    """@private"""
    name: str
    provider: ExternalModelProviderName
    provider_data: Optional['ModelProviderDataInput'] = Field(alias='providerData', default=None)
    description: Optional[str] = None

class AddHFModelInput(BaseModel):
    """@private"""
    model_id: str = Field(alias='modelId')
    output_model_name: str = Field(alias='outputModelName')
    output_model_key: Optional[str] = Field(alias='outputModelKey', default=None)
    hf_token: str = Field(alias='hfToken')
    compute_pool: Optional[str] = Field(alias='computePool', default=None)
    num_gpus: int = Field(alias='numGpus', default=0)

class AddModelInput(BaseModel):
    """@private"""
    path: str
    name: str
    key: Optional[str] = None

class AnthropicProviderDataInput(BaseModel):
    """@private"""
    api_key: str = Field(alias='apiKey')
    external_model_id: str = Field(alias='externalModelId')

class ApiKeyCreate(BaseModel):
    """@private"""
    user: str

class ArtifactFilter(BaseModel):
    """@private"""
    kinds: Optional[List[JobArtifactKind]] = None
    job_id: Optional[Any] = Field(alias='jobId', default=None)

class AttachModel(BaseModel):
    """@private"""
    use_case: str = Field(alias='useCase')
    model: str
    attached: bool = True
    placement: Optional['ModelPlacementInput'] = None
    wait: bool = False
    'Wait for the model to be deployed or not'

class CancelAllocationInput(BaseModel):
    """@private"""
    harmony_group: str = Field(alias='harmonyGroup')
    job_id: str = Field(alias='jobId')

class CapabilityFilter(BaseModel):
    """@private"""
    any: Optional[List[ModelCapabilityFilter]] = None
    all: Optional[List[ModelCapabilityFilter]] = None

class CompletionComparisonFilterInput(BaseModel):
    """@private"""
    metric: str

class CompletionFeedbackFilterInput(BaseModel):
    """@private"""
    metric: str
    gt: Optional[float] = None
    gte: Optional[float] = None
    eq: Optional[float] = None
    neq: Optional[float] = None
    lt: Optional[float] = None
    lte: Optional[float] = None
    reasons: Optional[List[str]] = None
    user: Optional[Any] = None

class CompletionLabelValue(BaseModel):
    """@private"""
    key: str
    value: str

class CompletionsByFilters(BaseModel):
    """@private"""
    filters: 'ListCompletionsFilterInput'
    exclude: List[Any]

class CompletionsById(BaseModel):
    """@private"""
    include: List[Any]

class CreateRecipeInput(BaseModel):
    """@private"""
    name: str
    key: Optional[str] = None
    description: Optional[str] = None
    labels: Optional[List['LabelInput']] = None

class CreateToolProviderInput(BaseModel):
    """@private"""
    key: str
    name: str
    uri: str
    protocol: Protocol

class CursorPageInput(BaseModel):
    """@private"""
    first: Optional[int] = None
    after: Optional[str] = None
    before: Optional[str] = None
    last: Optional[int] = None
    offset: Optional[int] = None

class CustomConfigInput(BaseModel):
    """@private"""
    description: Optional[str] = None

class CustomRecipeFilterInput(BaseModel):
    """@private"""
    labels: Optional[List['LabelFilter']] = None

class DatasetCompletionQuery(BaseModel):
    """@private"""
    from_selection: Optional['CompletionsById'] = Field(alias='fromSelection', default=None)
    from_filters: Optional['CompletionsByFilters'] = Field(alias='fromFilters', default=None)
    from_groups: Optional['FromGroupsQuery'] = Field(alias='fromGroups', default=None)

class DatasetCreate(BaseModel):
    """@private"""
    use_case: str = Field(alias='useCase')
    name: str
    key: Optional[str] = None
    source: Optional[DatasetSource] = None

class DatasetCreateFromFilters(BaseModel):
    """@private"""
    use_case: str = Field(alias='useCase')
    name: str
    key: Optional[str] = None
    completion_query: 'DatasetCompletionQuery' = Field(alias='completionQuery')
    sample_config: Optional['SampleConfig'] = Field(alias='sampleConfig', default=None)
    feedback_filters: Optional['FeedbackFilterInput'] = Field(alias='feedbackFilters', default=None)
    kind: DatasetKind
    metrics: Optional[List[str]] = None

class DatasetCreateFromMultipartUpload(BaseModel):
    """@private"""
    use_case: str = Field(alias='useCase')
    name: str
    key: Optional[str] = None
    source: Optional[DatasetSource] = None
    upload_session_id: str = Field(alias='uploadSessionId')

class DatasetUploadProcessingStatusInput(BaseModel):
    """@private"""
    use_case: str = Field(alias='useCase')
    dataset_id: Any = Field(alias='datasetId')

class EmojiInput(BaseModel):
    """@private"""
    native: str

class FeedbackAddInput(BaseModel):
    """@private"""
    value: Any
    details: Optional[str] = None
    reason: Optional[str] = None
    user_id: Optional[Any] = Field(alias='userId', default=None)

class FeedbackFilterInput(BaseModel):
    """@private"""
    labels: Optional[List['LabelFilter']] = None

class FeedbackUpdateInput(BaseModel):
    """@private"""
    value: Optional[Any] = None
    details: Optional[str] = None

class FromGroupsQuery(BaseModel):
    """@private"""
    filters: 'ListCompletionsFilterInput'
    grouping: CompletionGroupBy
    groups: List['GroupSelectionQuery']

class GlobalUsageFilterInput(BaseModel):
    """@private"""
    timerange: Optional['TimeRange'] = None
    'use none to get "all time" data'
    interval: DateBucketUnit
    timezone: Optional[str] = None

class GoogleProviderDataInput(BaseModel):
    """@private"""
    api_key: str = Field(alias='apiKey')
    external_model_id: str = Field(alias='externalModelId')

class GraderConfigInput(BaseModel):
    """@private"""
    judge: Optional['JudgeConfigInput'] = None
    prebuilt: Optional['PrebuiltConfigInput'] = None
    remote: Optional['RemoteConfigInput'] = None
    custom: Optional['CustomConfigInput'] = None

class GraderCreateInput(BaseModel):
    """@private"""
    name: str
    key: Optional[str] = None
    grader_type: GraderTypeEnum = Field(alias='graderType')
    grader_config: 'GraderConfigInput' = Field(alias='graderConfig')
    metric: Optional['MetricGetOrCreate'] = None

class GraderUpdateInput(BaseModel):
    """@private"""
    name: Optional[str] = None
    grader_type: Optional[GraderTypeEnum] = Field(alias='graderType', default=None)
    grader_config: Optional['GraderConfigInput'] = Field(alias='graderConfig', default=None)

class GroupSelection(BaseModel):
    """@private"""
    exclude: Optional[List[Any]] = None
    select_only: Optional[List[Any]] = Field(alias='selectOnly', default=None)

class GroupSelectionQuery(BaseModel):
    """@private"""
    group_id: str = Field(alias='groupId')
    selection: 'GroupSelection'

class JobArtifactFilter(BaseModel):
    """@private"""
    kinds: List[JobArtifactKind]

class JobInput(BaseModel):
    """@private"""
    recipe: str
    use_case: str = Field(alias='useCase')
    args: Any
    name: Optional[str] = None
    compute_pool: Optional[str] = Field(alias='computePool', default=None)
    num_gpus: int = Field(alias='numGpus')

class JudgeConfigInput(BaseModel):
    """@private"""
    model: str
    criteria: str
    examples: List['JudgeExampleInput']
    system_template: str = Field(alias='systemTemplate')
    user_template: str = Field(alias='userTemplate')

class JudgeCreate(BaseModel):
    """@private"""
    key: Optional[str] = None
    name: str
    criteria: str
    examples: List['JudgeExampleInput'] = Field(default_factory=lambda: [])
    model: str
    metric: Optional[str] = None

class JudgeExampleInput(BaseModel):
    """@private"""
    input: List['JudgeExampleInputTurnEntry']
    reasoning: Optional[str] = None
    output: str
    pass_: bool = Field(alias='pass')
    id: Optional[Any] = None

class JudgeExampleInputTurnEntry(BaseModel):
    """@private"""
    role: str
    content: str

class JudgeUpdate(BaseModel):
    """@private"""
    name: Optional[str] = None
    criteria: Optional[str] = None
    examples: Optional[List['JudgeExampleInput']] = None
    model: Optional[str] = None

class LabelFilter(BaseModel):
    """@private"""
    key: str
    value: Optional[List[str]] = None

class LabelInput(BaseModel):
    """@private"""
    key: str
    value: str

class ListCompletionsFilterInput(BaseModel):
    """@private"""
    use_case: str = Field(alias='useCase')
    models: Optional[List[str]] = None
    timerange: Optional['TimeRange'] = None
    session_id: Optional[Any] = Field(alias='sessionId', default=None)
    user_id: Optional[Any] = Field(alias='userId', default=None)
    feedbacks: Optional[List['CompletionFeedbackFilterInput']] = None
    comparisons: Optional[List['CompletionComparisonFilterInput']] = None
    labels: Optional[List['LabelFilter']] = None
    prompt_hash: Optional[str] = Field(alias='promptHash', default=None)
    completion_id: Optional[Any] = Field(alias='completionId', default=None)
    source: Optional[List[CompletionSource]] = None

class ListJobsFilterInput(BaseModel):
    """@private"""
    use_case: Optional[str] = Field(alias='useCase', default=None)
    kind: Optional[List[JobKind]] = None
    status: Optional[List[JobStatus]] = None
    timerange: Optional['TimeRange'] = None
    custom_recipes: Optional[List[str]] = Field(alias='customRecipes', default=None)
    artifacts: Optional['JobArtifactFilter'] = None

class MetricCreate(BaseModel):
    """@private"""
    name: str
    key: Optional[str] = None
    kind: MetricKind
    scoring_type: MetricScoringType = Field(alias='scoringType', default=MetricScoringType.HIGHER_IS_BETTER)
    description: Optional[str] = None
    unit: Optional[str] = None

class MetricGetOrCreate(BaseModel):
    """@private"""
    existing: Optional[str] = None
    new: Optional['MetricCreate'] = None

class MetricLink(BaseModel):
    """@private"""
    use_case: str = Field(alias='useCase')
    metric: str

class MetricTrendInput(BaseModel):
    """@private"""
    timerange: Optional['TimeRange'] = None
    aggregation: MetricAggregation = MetricAggregation.AVERAGE

class MetricUnlink(BaseModel):
    """@private"""
    use_case: str = Field(alias='useCase')
    metric: str

class ModelComputeConfigInput(BaseModel):
    """@private"""
    tp: Optional[int] = None
    kv_cache_len: Optional[int] = Field(alias='kvCacheLen', default=None)
    max_seq_len: Optional[int] = Field(alias='maxSeqLen', default=None)

class ModelFilter(BaseModel):
    """@private"""
    in_storage: Optional[bool] = Field(alias='inStorage', default=None)
    available: Optional[bool] = None
    trainable: Optional[bool] = None
    capabilities: Optional['CapabilityFilter'] = None
    view_all: Optional[bool] = Field(alias='viewAll', default=None)
    online: Optional[List[ModelOnline]] = None

class ModelPlacementInput(BaseModel):
    """@private"""
    compute_pools: List[str] = Field(alias='computePools')
    max_ttft_ms: Optional[int] = Field(alias='maxTtftMs', default=None)

class ModelProviderDataInput(BaseModel):
    """@private"""
    open_ai: Optional['OpenAIProviderDataInput'] = Field(alias='openAI', default=None)
    google: Optional['GoogleProviderDataInput'] = None
    anthropic: Optional['AnthropicProviderDataInput'] = None

class ModelServiceDisconnect(BaseModel):
    """@private"""
    use_case: str = Field(alias='useCase')
    model_service: str = Field(alias='modelService')

class ModelServiceFilter(BaseModel):
    """@private"""
    model: Optional[str] = None
    capabilities: Optional['CapabilityFilter'] = None

class OpenAIProviderDataInput(BaseModel):
    """@private"""
    api_key: str = Field(alias='apiKey')
    external_model_id: OpenAIModel = Field(alias='externalModelId')

class OrderPair(BaseModel):
    """@private"""
    field: str
    order: SortDirection

class PrebuiltConfigInput(BaseModel):
    """@private"""
    key: PrebuiltCriteriaKey
    model: str

class PrebuiltJudgeCreate(BaseModel):
    """@private"""
    key: Optional[str] = None
    name: str
    model: str
    prebuilt_criteria_key: PrebuiltCriteriaKey = Field(alias='prebuiltCriteriaKey')

class RemoteConfigInput(BaseModel):
    """@private"""
    url: str

class RemoteEnvCreate(BaseModel):
    """@private"""
    url: str
    key: Optional[str] = None
    name: Optional[str] = None
    description: Optional[str] = None

class ResizePartitionInput(BaseModel):
    """@private"""
    harmony_group: str = Field(alias='harmonyGroup')
    size: int

class RoleCreate(BaseModel):
    """@private"""
    key: Optional[str] = None
    name: str
    permissions: List[str]

class SampleConfig(BaseModel):
    """@private"""
    selection_type: SelectionTypeInput = Field(alias='selectionType')
    sample_size: Optional[int] = Field(alias='sampleSize', default=None)

class SearchInput(BaseModel):
    """@private"""
    query: str

class SystemPromptTemplateCreate(BaseModel):
    """@private"""
    name: str
    template: str

class SystemPromptTemplateUpdate(BaseModel):
    """@private"""
    system_prompt_template: Any = Field(alias='systemPromptTemplate')
    name: Optional[str] = None
    template: str
    update_model_services: bool = Field(alias='updateModelServices', default=False)

class TeamCreate(BaseModel):
    """@private"""
    key: Optional[str] = None
    name: str

class TeamMemberRemove(BaseModel):
    """@private"""
    user: str
    team: str

class TeamMemberSet(BaseModel):
    """@private"""
    user: str
    team: str
    role: str

class TimeRange(BaseModel):
    """@private"""
    from_: int | str = Field(alias='from')
    to: int | str

class TimeseriesInput(BaseModel):
    """@private"""
    interval: TimeseriesInterval
    timerange: Optional['TimeRange'] = None
    timezone: Optional[str] = None
    by_model: bool = Field(alias='byModel', default=False)
    aggregation: MetricAggregation = MetricAggregation.AVERAGE

class UnitConfigInput(BaseModel):
    """@private"""
    symbol: str
    position: UnitPosition

class UpdateCompletion(BaseModel):
    """@private"""
    id: Any
    remove_labels: Optional[List['CompletionLabelValue']] = Field(alias='removeLabels', default=None)
    'remove some label value. This operation is atomic'
    add_labels: Optional[List['CompletionLabelValue']] = Field(alias='addLabels', default=None)
    'add a label value. This operation is atomic'
    set_labels: Optional[List['CompletionLabelValue']] = Field(alias='setLabels', default=None)
    "set the completion labels to this list. If you want to only add or remove specific labels,\nit's better to use `add_labels` or `remove_labels`"
    metadata: Optional[Any] = None
    'set metadata associated with this prompt for use with external reward servers'

class UpdateModelService(BaseModel):
    """@private"""
    use_case: str = Field(alias='useCase')
    model_service: str = Field(alias='modelService')
    is_default: Optional[bool] = Field(alias='isDefault', default=None)
    attached: Optional[bool] = None
    desired_online: Optional[bool] = Field(alias='desiredOnline', default=None)
    name: Optional[str] = None
    system_prompt_template: Optional[Any] = Field(alias='systemPromptTemplate', default=None)
    placement: Optional['ModelPlacementInput'] = None
    tool_providers: Optional[List[str]] = Field(alias='toolProviders', default=None)

class UpdateRecipeInput(BaseModel):
    """@private"""
    name: Optional[str] = None
    description: Optional[str] = None
    labels: Optional[List['LabelInput']] = None

class UpdateToolProviderInput(BaseModel):
    """@private"""
    name: Optional[str] = None
    uri: Optional[str] = None
    protocol: Optional[Protocol] = None

class UsageFilterInput(BaseModel):
    """@private"""
    model_id: Any = Field(alias='modelId')
    timerange: Optional['TimeRange'] = None
    unit: DateBucketUnit
    timezone: Optional[str] = None

class UsagePerUseCaseFilterInput(BaseModel):
    """@private"""
    model_id: Any = Field(alias='modelId')
    timerange: Optional['TimeRange'] = None

class UseCaseCreate(BaseModel):
    """@private"""
    name: str
    team: Optional[str] = None
    key: Optional[str] = None
    description: Optional[str] = None
    gradient_color: Optional[str] = Field(alias='gradientColor', default=None)
    metadata: Optional['UseCaseMetadataInput'] = None
    settings: Optional['UseCaseSettingsInput'] = None

class UseCaseFilter(BaseModel):
    """@private"""
    is_archived: Optional[bool] = Field(alias='isArchived', default=None)

class UseCaseMetadataInput(BaseModel):
    """@private"""
    emoji: Optional['EmojiInput'] = None

class UseCaseSettingsInput(BaseModel):
    """@private"""
    default_metric: Optional[str] = Field(alias='defaultMetric', default=None)

class UseCaseShareInput(BaseModel):
    """@private"""
    team: str
    role: str
    is_owner: bool = Field(alias='isOwner')

class UseCaseShares(BaseModel):
    """@private"""
    shares: List['UseCaseShareInput']

class UseCaseUpdate(BaseModel):
    """@private"""
    name: Optional[str] = None
    description: Optional[str] = None
    widgets: Optional[List['WidgetInput']] = None
    metadata: Optional['UseCaseMetadataInput'] = None
    settings: Optional['UseCaseSettingsInput'] = None
    is_archived: Optional[bool] = Field(alias='isArchived', default=None)

class UserCreate(BaseModel):
    """@private"""
    email: str
    name: str
    teams: List['UserCreateTeamWithRole']

class UserCreateTeamWithRole(BaseModel):
    """@private"""
    team: str
    role: str

class WidgetInput(BaseModel):
    """@private"""
    title: str
    metric: str
    aggregation: MetricAggregation
    unit: 'UnitConfigInput'
AddExternalModelInput.model_rebuild()
AttachModel.model_rebuild()
CompletionsByFilters.model_rebuild()
CreateRecipeInput.model_rebuild()
CustomRecipeFilterInput.model_rebuild()
DatasetCompletionQuery.model_rebuild()
DatasetCreateFromFilters.model_rebuild()
FeedbackFilterInput.model_rebuild()
FromGroupsQuery.model_rebuild()
GlobalUsageFilterInput.model_rebuild()
GraderConfigInput.model_rebuild()
GraderCreateInput.model_rebuild()
GraderUpdateInput.model_rebuild()
GroupSelectionQuery.model_rebuild()
JudgeConfigInput.model_rebuild()
JudgeCreate.model_rebuild()
JudgeExampleInput.model_rebuild()
JudgeUpdate.model_rebuild()
ListCompletionsFilterInput.model_rebuild()
ListJobsFilterInput.model_rebuild()
MetricGetOrCreate.model_rebuild()
MetricTrendInput.model_rebuild()
ModelFilter.model_rebuild()
ModelProviderDataInput.model_rebuild()
ModelServiceFilter.model_rebuild()
TimeseriesInput.model_rebuild()
UpdateCompletion.model_rebuild()
UpdateModelService.model_rebuild()
UpdateRecipeInput.model_rebuild()
UsageFilterInput.model_rebuild()
UsagePerUseCaseFilterInput.model_rebuild()
UseCaseCreate.model_rebuild()
UseCaseMetadataInput.model_rebuild()
UseCaseShares.model_rebuild()
UseCaseUpdate.model_rebuild()
UserCreate.model_rebuild()
WidgetInput.model_rebuild()


from io import IOBase
from pydantic import BaseModel as PydanticBaseModel
from pydantic import ConfigDict

class UnsetType:
    """@private"""

    def __bool__(self) -> bool:
        return False
UNSET = UnsetType()

class BaseModel(PydanticBaseModel):
    """@private"""
    model_config = ConfigDict(populate_by_name=True, validate_assignment=True, arbitrary_types_allowed=True, protected_namespaces=())

class Upload:
    """@private"""

    def __init__(self, filename: str, content: IOBase, content_type: str):
        self.filename = filename
        self.content = content
        self.content_type = content_type